/*
 * ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2001 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Apache Software Foundation (http://www.apache.org/)."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" and
 * "Apache JMeter" must not be used to endorse or promote products
 * derived from this software without prior written permission. For
 * written permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 * "Apache JMeter", nor may "Apache" appear in their name, without
 * prior written permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */
package org.apache.jmeter.config;
import java.io.*;

import java.util.*;

import org.apache.jmeter.gui.JMeterComponentModel;
import org.apache.jmeter.save.*;
import org.apache.jmeter.save.xml.TagHandler;

/************************************************************
 *  Title: JMeter Description: Copyright: Copyright (c) 2000 Company: Apache
 *
 *@author     Michael Stover
 *@created    $Date: 2001/08/30 17:19:01 $
 *@version    1.0
 ***********************************************************/

public abstract class AbstractConfigElement implements ConfigElement,
		JMeterComponentModel, Saveable, Serializable
{

	protected Map properties = new HashMap();
	public final static String NAME = "name";

	private static List addableList = new LinkedList();

	/************************************************************
	 *  Sets the name of the config element.
	 *
	 *@param  name  !ToDo (Parameter description)
	 ***********************************************************/
	public void setName(String name)
	{
		properties.put(NAME, name);
	}

	/************************************************************
	 *  Gets the GuiElements attribute of the AbstractConfigElement object
	 *
	 *@return    The GuiElements value
	 ***********************************************************/
	public Collection getGuiElements()
	{
		return Collections.EMPTY_LIST;
	}

	/************************************************************
	 *  Gets the TagHandler that will handle the parsing of the xml for this type
	 *  of object.
	 *
	 *@return    The TagHandlerClass value
	 ***********************************************************/
	public Class getTagHandlerClass()
	{
		return org.apache.jmeter.save.handlers.AbstractConfigElementHandler.class;
	}


	/************************************************************
	 *  Gets the name of the config element.
	 *
	 *@return    !ToDo (Return description)
	 ***********************************************************/
	public String getName()
	{
		return (String)properties.get(NAME);
	}

	/************************************************************
	 *  Gets the Editable attribute of the AbstractConfigElement object
	 *
	 *@return    The Editable value
	 ***********************************************************/
	public boolean isEditable()
	{
		return true;
	}

	/************************************************************
	 *  Gets the AddList attribute of the AbstractConfigElement object
	 *
	 *@return    The AddList value
	 ***********************************************************/
	public Collection getAddList()
	{
		return addableList;
	}

	/************************************************************
	 *  Gets a property of the config element.
	 *
	 *@param  key  !ToDo (Parameter description)
	 *@return      !ToDo (Return description)
	 ***********************************************************/
	public Object getProperty(String key)
	{
		return properties.get(key);
	}

	/************************************************************
	 *  Get the names of all valid properties for this config element.
	 *
	 *@return    !ToDo (Return description)
	 ***********************************************************/
	public Collection getPropertyNames()
	{
		return properties.keySet();
	}

	/************************************************************
	 *  Gets the GuiClass attribute of the AbstractConfigElement object
	 *
	 *@return    The GuiClass value
	 ***********************************************************/
	public Class getGuiClass()
	{
		return org.apache.jmeter.config.gui.AbstractConfigGui.class;
	}

	/************************************************************
	 *  Description of the Method
	 ***********************************************************/
	public void uncompile()
	{
	}

	/************************************************************
	 *  Description of the Method
	 *
	 *@return    Description of the Returned Value
	 ***********************************************************/
	public boolean expectsModification()
	{
		return false;
	}

	/************************************************************
	 *  Sets a property of the config element.
	 *
	 *@param  key    !ToDo (Parameter description)
	 *@param  value  !ToDo (Parameter description)
	 ***********************************************************/
	public void putProperty(String key, Object value)
	{
		properties.put(key, value);
	}

	/************************************************************
	 *  !ToDo
	 *
	 *@param  config  !ToDo
	 ***********************************************************/
	public abstract void addConfigElement(ConfigElement config);

	public void addJMeterComponent(JMeterComponentModel child)
	{
		if(child instanceof ConfigElement)
		{
			addConfigElement((ConfigElement)child);
		}
	}

	/************************************************************
	 *  !ToDo (Method description)
	 *
	 *@return    !ToDo (Return description)
	 ***********************************************************/
	public abstract Object clone();

	/************************************************************
	 *  !ToDo (Method description)
	 *
	 *@param  key  !ToDo (Parameter description)
	 *@return      !ToDo (Return description)
	 ***********************************************************/
	protected boolean contains(String key)
	{
		Object value = properties.get(key);
		if(value == null || value.toString().equals(""))
		{
			return false;
		}
		else
		{
			return true;
		}
	}

	/************************************************************
	 *  !ToDo (Method description)
	 *
	 *@param  config  !ToDo (Parameter description)
	 ***********************************************************/
	protected void updatePropertyIfAbsent(AbstractConfigElement config)
	{
		for(Iterator it = config.getPropertyNames().iterator(); it.hasNext(); )
		{
			String propName = (String)it.next();
			if(!contains(propName) || getProperty(propName) == null ||
					getProperty(propName).toString().equals(""))
			{
				properties.put(propName, config.getProperty(propName));
			}
		}
	}

	/************************************************************
	 *  !ToDo (Method description)
	 *
	 *@param  newConfig  !ToDo (Parameter description)
	 ***********************************************************/
	protected void configureClone(AbstractConfigElement newConfig)
	{
		newConfig.properties = (Map)((HashMap)properties).clone();
	}

	private void copyProperties(Map props)
	{
		properties.putAll(props);
	}
}
