/*
 * ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2001 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Apache Software Foundation (http://www.apache.org/)."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" and
 * "Apache JMeter" must not be used to endorse or promote products
 * derived from this software without prior written permission. For
 * written permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 * "Apache JMeter", nor may "Apache" appear in their name, without
 * prior written permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */
package org.apache.jmeter.config;

import java.util.*;
import java.io.*;

import org.apache.jmeter.gui.JMeterComponentModel;
import org.apache.jmeter.save.Saveable;
import org.apache.jmeter.util.JMeterUtils;

/************************************************************
 *  Title: Apache JMeter Description: Copyright: Copyright (c) 2000 Company:
 *  Apache Foundation
 *
 *@author     Michael Stover
 *@created    $Date: 2002/02/23 01:21:06 $
 *@version    1.0
 ***********************************************************/

public class Arguments implements JMeterComponentModel, Saveable,Serializable,Cloneable
{
	public static String[] COLUMN_NAMES = {
	  JMeterUtils.getResString("name"), 
	  JMeterUtils.getResString("value")
	};

	private List args;
	private String name;

	/************************************************************
	 *  !ToDo (Constructor description)
	 ***********************************************************/
	public Arguments()
	{
		args = new ArrayList();
	}

	/************************************************************
	 *  !ToDo (Method description)
	 *
	 *@param  newName  !ToDo (Parameter description)
	 ***********************************************************/
	public void setName(String newName)
	{
		name = newName;
	}

	public void addJMeterComponent(JMeterComponentModel child)
	{
	}

	public Object clone()
	{
		Arguments newArgs = new Arguments();
		Iterator iter = args.iterator();
		while (iter.hasNext())
		{
			Argument item = (Argument)iter.next();
			newArgs.addArgument(item.getName(),item.getValue());
		}

		return newArgs;
	}

	/************************************************************
	 *  Gets the TagHandlerClass attribute of the Arguments object
	 *
	 *@return    The TagHandlerClass value
	 ***********************************************************/
	public Class getTagHandlerClass()
	{
		return org.apache.jmeter.save.handlers.ArgumentsHandler.class;
	}

	/************************************************************
	 *  !ToDoo (Method description)
	 *
	 *@return    !ToDo (Return description)
	 ***********************************************************/
	public String getClassLabel()
	{
		return "Argument List";
	}

	/************************************************************
	 *  !ToDoo (Method description)
	 *
	 *@return    !ToDo (Return description)
	 ***********************************************************/
	public Class getGuiClass()
	{
		return org.apache.jmeter.config.gui.ArgumentsPanel.class;
	}

	/************************************************************
	 *  !ToDoo (Method description)
	 *
	 *@return    !ToDo (Return description)
	 ***********************************************************/
	public String getName()
	{
		return name;
	}

	/************************************************************
	 *  !ToDoo (Method description)
	 *
	 *@return    !ToDo (Return description)
	 ***********************************************************/
	public boolean isEditable()
	{
		return true;
	}

	/************************************************************
	 *  !ToDoo (Method description)
	 *
	 *@return    !ToDo (Return description)
	 ***********************************************************/
	public Collection getAddList()
	{
		return null;
	}

	/************************************************************
	 *  Description of the Method
	 ***********************************************************/
	public void uncompile()
	{
	}

	/************************************************************
	 *  !ToDo
	 *
	 *@param  name   !ToDo
	 *@param  value  !ToDo
	 ***********************************************************/
	public void addArgument(String name, Object value)
	{
		args.add(new Argument(name, value));
	}

	/************************************************************
	 *  !ToDo (Method description)
	 *
	 *@return    !ToDo (Return description)
	 ***********************************************************/
	public Iterator iterator()
	{
		return args.iterator();
	}

	public String toString()
	{
		StringBuffer str = new StringBuffer();
		Iterator iter = args.iterator();
		while (iter.hasNext())
		{
			Argument arg = (Argument)iter.next();
			str.append(arg.name+"="+arg.value);
			if (iter.hasNext())
			{
				str.append("&");
			}
		}
		return str.toString();
	}

	public void removeArgument(int row)
	{
		if (row < args.size())
		{
			args.remove(row);
		}
	}

	public void removeArgument(Argument arg)
	{
		args.remove(arg);
	}

	public void removeAllArguments()
	{
		if (args.size() > 0)
		{
			args.clear();
		}
	}

	public void addEmptyArgument()
	{
		args.add(new Argument("",""));
	}

	public int getArgumentCount()
	{
		return args.size();
	}

	public Argument getArgument(int row)
	{
		Argument argument = null;

		if (row < args.size())
		{
			argument = (Argument)args.get(row);
		}

		return argument;
	}
}
