/*
 * ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2001 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Apache Software Foundation (http://www.apache.org/)."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" and
 * "Apache JMeter" must not be used to endorse or promote products
 * derived from this software without prior written permission. For
 * written permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 * "Apache JMeter", nor may "Apache" appear in their name, without
 * prior written permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */
package org.apache.jmeter.control;
import java.util.*;
import java.io.*;
import org.apache.jmeter.config.ConfigElement;
import org.apache.jmeter.gui.*;
import org.apache.jmeter.samplers.*;
import org.apache.jmeter.samplers.SampleResult;
import org.apache.jmeter.save.Saveable;
import org.apache.jmeter.assertions.Assertion;

/************************************************************
 *  Title: JMeter Description: Copyright: Copyright (c) 2000 Company: Apache
 *
 *@author     Michael Stover
 *@created    $Date: 2002/02/19 00:05:35 $
 *@version    1.0
 ***********************************************************/

public class GenericController implements SamplerController, JMeterComponentModel,
		Saveable, Serializable
{
	protected List subControllers = new LinkedList();
	protected SamplerController current;
	protected Iterator controlIt;
	private List configs = new LinkedList();
	private boolean returnedNull = false;
	private boolean done = false, timeForNext = false;
	private String name;
	private List assertions = new LinkedList();

	private static List addableList = null;

	/************************************************************
	 *  !ToDo (Constructor description)
	 ***********************************************************/
	public GenericController()
	{
		name = getClassLabel();
	}

	/************************************************************
	 *  !ToDo (Method description)
	 *
	 *@param  name  !ToDo (Parameter description)
	 ***********************************************************/
	public void setName(String name)
	{
		this.name = name;
	}

	/************************************************************
	 *  Gets the TagHandlerClass attribute of the GenericController object
	 *
	 *@return    The TagHandlerClass value
	 ***********************************************************/
	public Class getTagHandlerClass()
	{
		return org.apache.jmeter.save.handlers.GenericControllerHandler.class;
	}

	/************************************************************
	 *  Gets the ConfigElements attribute of the GenericController object
	 *
	 *@return    The ConfigElements value
	 ***********************************************************/
	public List getConfigElements()
	{
		return configs;
	}

	/************************************************************
	 *  Gets the SubControllers attribute of the GenericController object
	 *
	 *@return    The SubControllers value
	 ***********************************************************/
	public List getSubControllers()
	{
		return subControllers;
	}

	/************************************************************
	 *  Gets the GuiClass attribute of the GenericController object
	 *
	 *@return    The GuiClass value
	 ***********************************************************/
	public Class getGuiClass()
	{
		return org.apache.jmeter.gui.NamePanel.class;
	}

	/************************************************************
	 *  Gets the ClassLabel attribute of the GenericController object
	 *
	 *@return    The ClassLabel value
	 ***********************************************************/
	public String getClassLabel()
	{
		return "Default Controller";
	}

	/************************************************************
	 *  Gets the Editable attribute of the GenericController object
	 *
	 *@return    The Editable value
	 ***********************************************************/
	public boolean isEditable()
	{
		return true;
	}

	/************************************************************
	 *  Gets the AddList attribute of the GenericController object
	 *
	 *@return    The AddList value
	 ***********************************************************/
	public Collection getAddList()
	{
		if (addableList == null)
		{
			addableList = new LinkedList();
			addableList.add(MenuFactory.getLogicControllerMenu());
			addableList.add(MenuFactory.getGenerativeControllerMenu());
			addableList.add(MenuFactory.getConfigElementMenu());
		}
		return addableList;
	}

	/************************************************************
	 *  !ToDoo (Method description)
	 *
	 *@return    !ToDo (Return description)
	 ***********************************************************/
	public String getName()
	{
		return name;
	}

	/************************************************************
	 *  this method is ignored.
	 *
	 *@return    !ToDo (Return description)
	 ***********************************************************/
	public Collection getListeners()
	{
		Collection listeners = new LinkedList();
		Iterator iter = subControllers.iterator();
		while (iter.hasNext())
		{
			SamplerController item = (SamplerController)iter.next();
			listeners.addAll(item.getListeners());
		}
		addSelfOrNot(listeners);
		return listeners;
	}

	public List getAssertions()
	{
		return assertions;
	}

	/************************************************************
	 *  !ToDo
	 *
	 *@param  assert  !ToDo
	 ***********************************************************/
	public void addAssertion(Assertion assertion)
	{
		assertions.add(assertion);
	}

	/************************************************************
	 *  !ToDo
	 *
	 *@param  child  !ToDo
	 ***********************************************************/
	public void addJMeterComponent(JMeterComponentModel child)
	{
		if (child instanceof SamplerController)
		{
			addSamplerController((SamplerController)child);
		}
		else if (child instanceof ConfigElement)
		{
			addConfigElement((ConfigElement)child);
		}
		else if (child instanceof Assertion)
		{
			addAssertion((Assertion)child);
		}
	}

	/************************************************************
	 *  Description of the Method
	 ***********************************************************/
	public void uncompile()
	{
		subControllers.clear();
		configs.clear();
		assertions.clear();
		current = null;
		controlIt = null;
		returnedNull = false;
		done = false;
		timeForNext = false;
	}

	/************************************************************
	 *  Retrieves the next Entry to be sampled.
	 *
	 *@return    !ToDo (Return description)
	 ***********************************************************/
	public Entry nextEntry()
	{
		Entry entry = null;
		if (!init())
		{
			return null;
		}
		entry = current.nextEntry();
		if (entry == null)
		{
			return removeFromIterator();
		}
		else if (entry instanceof CycleEntry)
		{
			return endEntryCycle();
		}
		else
		{
			configureEntry(entry);
		}
		return entry;
	}


	/************************************************************
	 *  !ToDo (Method description)
	 *
	 *@return    !ToDo (Return description)
	 ***********************************************************/
	public Collection allEntries()
	{
		return null;
	}

	/************************************************************
	 *  !ToDo
	 *
	 *@param  controller  !ToDo
	 ***********************************************************/
	public void addSamplerController(SamplerController controller)
	{
		subControllers.add(controller);
	}

	/************************************************************
	 *  !ToDo
	 *
	 *@param  config  !ToDo
	 ***********************************************************/
	public void addConfigElement(ConfigElement config)
	{
		configs.add(config);
	}

	/************************************************************
	 *  !ToDo (Method description)
	 *
	 *@return    !ToDo (Return description)
	 ***********************************************************/
	public Object clone()
	{
		GenericController control = new GenericController();
		standardCloneProc(control);
		return control;
	}

	/************************************************************
	 *  Easy method to override if your controller wishes to add itself as a
	 *  listener. Simply call listeners.add(this), and your controller will be
	 *  added, and sub-controllers will also be given a chance to add themselves.
	 *  This is easier than overriding getListeners() and having to replicate that
	 *  logic.
	 *
	 *@param  listeners  The feature to be added to the SelfOrNot attribute
	 ***********************************************************/
	protected void addSelfOrNot(Collection listeners)
	{
	}

	/************************************************************
	 *  Description of the Method
	 *
	 *@return    Description of the Returned Value
	 ***********************************************************/
	protected Entry removeFromIterator()
	{
		controlIt.remove();
		return endEntryCycle();
	}

	/************************************************************
	 *  Description of the Method
	 *
	 *@return    Description of the Returned Value
	 ***********************************************************/
	protected Entry endEntryCycle()
	{
		if (controlIt != null && controlIt.hasNext())
		{
			current = (SamplerController)controlIt.next();
		}
		else
		{
			controlIt = null;
			return new CycleEntry();
		}
		return nextEntry();
	}

	/************************************************************
	 *  !ToDo (Method description)
	 *
	 *@param  control  !ToDo (Parameter description)
	 ***********************************************************/
	protected void standardCloneProc(SamplerController control)
	{
		Iterator iter = this.getConfigElements().iterator();
		while (iter.hasNext())
		{
			control.addConfigElement((ConfigElement)((ConfigElement)iter.next()).clone());
		}
		iter = subControllers.iterator();
		while (iter.hasNext())
		{
			control.addSamplerController((SamplerController)((SamplerController)iter.next()).clone());
		}
		iter = assertions.iterator();
		while(iter.hasNext())
		{
			((GenericController)control).addAssertion((Assertion)iter.next());
		}
		//control.setName(getName());
	}

	/************************************************************
	 *  Description of the Method
	 *
	 *@return    Description of the Returned Value
	 ***********************************************************/
	protected boolean init()
	{
		if (controlIt == null)
		{
			controlIt = subControllers.iterator();
			if (controlIt.hasNext())
			{
				current = (SamplerController)controlIt.next();
			}
			else
			{
				return false;
			}
		}
		return true;
	}

	/************************************************************
	 *  !ToDo
	 *
	 *@param  entry  !ToDo
	 ***********************************************************/
	protected void addAssertions(Entry entry)
	{
		if (entry != null)
		{
			Iterator iter = assertions.iterator();
			while (iter.hasNext())
			{
				entry.addAssertion((Assertion)iter.next());
			}
		}
	}

	/************************************************************
	 *  Adds a feature to the ConfigElements attribute of the GenericController
	 *  object
	 *
	 *@param  entry  The feature to be added to the ConfigElements attribute
	 ***********************************************************/
	protected void addConfigElements(Entry entry)
	{
		if (entry != null)
		{
			Iterator iter = configs.iterator();
			while (iter.hasNext())
			{
				entry.addConfigElement((ConfigElement)iter.next());
			}
		}
	}

	/************************************************************
	 *  !ToDo (Method description)
	 *
	 *@param  entry  !ToDo (Parameter description)
	 ***********************************************************/
	protected void configureEntry(Entry entry)
	{
		addConfigElements(entry);
		addAssertions(entry);
	}
}
