/*
 * ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2001 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Apache Software Foundation (http://www.apache.org/)."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" and
 * "Apache JMeter" must not be used to endorse or promote products
 * derived from this software without prior written permission. For
 * written permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 * "Apache JMeter", nor may "Apache" appear in their name, without
 * prior written permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */
package org.apache.jmeter.control;

import java.util.*;
import java.io.*;

import org.apache.jmeter.config.ConfigElement;
import org.apache.jmeter.samplers.*;
import org.apache.jmeter.util.JMeterUtils;

/************************************************************
 *  Title: Description: Copyright: Copyright (c) 2001 Company:
 *
 *@author     Michael Stover
 *@created    March 13, 2001
 *@version    1.0
 ***********************************************************/

public class InterleaveControl extends LogicController implements Serializable
{

	private boolean interleave;

	/************************************************************
	 *  Constructor for the InterleaveControl object
	 ***********************************************************/
	public InterleaveControl()
	{
		interleave = false;
	}

	public void uncompile()
	{
		super.uncompile();
		interleave = false;
	}

	/************************************************************
	 *  Gets the ClassLabel attribute of the InterleaveControl object
	 *
	 *@return    The ClassLabel value
	 ***********************************************************/
	public String getClassLabel()
	{
		return JMeterUtils.getResString("interleave_control_title");
	}

	/************************************************************
	 *  !ToDo (Method description)
	 *
	 *@return                       !ToDo (Return description)
	 ***********************************************************/
	public Entry nextEntry()
	{
		if(interleave)
		{
			interleave = !interleave;
			return new CycleEntry();
		}
		interleave = !interleave;
		Entry entry = null;
		do
		{
			entry = super.nextEntry();
			if(entry == null)
			{
				return null;
			}
		}while(entry instanceof CycleEntry);
		return entry;
	}

	/************************************************************
	 *  Description of the Method
	 *
	 *@return    Description of the Returned Value
	 ***********************************************************/
	protected Entry endEntryCycle()
	{
		if (controlIt != null && controlIt.hasNext())
		{
			current = (SamplerController)controlIt.next();
		}
		else
		{
			controlIt = null;
		}
		return new CycleEntry();
	}


	/************************************************************
	 *@return    The GuiClass value
	 ***********************************************************/
	public Class getGuiClass()
	{
		return org.apache.jmeter.control.gui.InterleaveControlGui.class;
	}

	/************************************************************
	 *  !ToDo (Method description)
	 *
	 *@return    !ToDo (Return description)
	 ***********************************************************/
	public Object clone()
	{
		InterleaveControl control = new InterleaveControl();
		Iterator iter = this.getConfigElements().iterator();
		while(iter.hasNext())
		{
			control.addConfigElement((ConfigElement)iter.next());
		}
		standardCloneProc(control);
		return control;
	}
}
