/*
 * ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2001 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Apache Software Foundation (http://www.apache.org/)."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" and
 * "Apache JMeter" must not be used to endorse or promote products
 * derived from this software without prior written permission. For
 * written permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 * "Apache JMeter", nor may "Apache" appear in their name, without
 * prior written permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */
package org.apache.jmeter.control;

import java.util.*;
import java.io.*;
import org.apache.jmeter.config.ConfigElement;

import org.apache.jmeter.gui.*;
import org.apache.jmeter.gui.tree.*;
import org.apache.jmeter.save.Saveable;
import org.apache.jmeter.save.handlers.TestPlanHandler;
import org.apache.jmeter.threads.ThreadGroup;
import org.apache.jmeter.util.JMeterUtils;
/************************************************************
 *  Title: JMeter Description: Copyright: Copyright (c) 2000 Company: Apache
 *
 *@author     Michael Stover
 *@created    March 13, 2001
 *@version    1.0
 ***********************************************************/

public class TestPlan implements JMeterComponentModel, Saveable,Serializable
{

	private String name;
	private List threadGroups = new LinkedList();
	private List configs = new LinkedList();
	private static TestPlan plan;
	private static List itemsCanAdd = new LinkedList();

	static
	{
		// WARNING! This String value must be identical to the String value returned
		// in org.apache.jmeter.threads.ThreadGroup.getClassLabel() method.
		// If it's not you will not be able to add a Thread Group element to a Test Plan.
		itemsCanAdd.add(JMeterUtils.getResString("threadgroup"));
	}

	private TestPlan()
	{
		name = "Test Plan";
	}

	private TestPlan(String name)
	{
		this.name = name;
	}

	/************************************************************
	 *  Sets the Name attribute of the TestPlan object
	 *
	 *@param  name  The new Name value
	 ***********************************************************/
	public void setName(String name)
	{
		this.name = name;
	}

	/************************************************************
	 *  Gets the TagHandlerClass attribute of the TestPlan object
	 *
	 *@return    The TagHandlerClass value
	 ***********************************************************/
	public Class getTagHandlerClass()
	{
		return org.apache.jmeter.save.handlers.TestPlanHandler.class;
	}

	/************************************************************
	 *  Gets the Name attribute of the TestPlan object
	 *
	 *@return    The Name value
	 ***********************************************************/
	public String getName()
	{
		return name;
	}

	public void addJMeterComponent(JMeterComponentModel child)
	{
		if (child instanceof ThreadGroup)
		{
			addThreadGroup((ThreadGroup)child);
		}
	}

	/************************************************************
	 *  Gets the ClassLabel attribute of the TestPlan object
	 *
	 *@return    The ClassLabel value
	 ***********************************************************/
	public String getClassLabel()
	{
		return "Test Plan";
	}

	/************************************************************
	 *  Gets the ThreadGroups attribute of the TestPlan object
	 *
	 *@return    The ThreadGroups value
	 ***********************************************************/
	public Collection getThreadGroups()
	{
		return threadGroups;
	}

	/************************************************************
	 *  Gets the ConfigElements attribute of the TestPlan object
	 *
	 *@return    The ConfigElements value
	 ***********************************************************/
	public Collection getConfigElements()
	{
		return configs;
	}

	/************************************************************
	 *  Indicates whether the item is editable. If so, the popup will include a
	 *  "edit" choice which will display the item's gui screen in the main panel.
	 *
	 *@return    The Editable value
	 ***********************************************************/
	public boolean isEditable()
	{
		return false;
	}

	/************************************************************
	 *  Gets the GuiClass attribute of the TestPlan object
	 *
	 *@return    The GuiClass value
	 ***********************************************************/
	public Class getGuiClass()
	{
		return javax.swing.JPanel.class;
	}

	/************************************************************
	 *  Returns the list of object types that can be added to this item.
	 *
	 *@return    The AddList value
	 ***********************************************************/
	public Collection getAddList()
	{
		return itemsCanAdd;
	}

	/************************************************************
	 *  Adds a feature to the ConfigElement attribute of the TestPlan object
	 *
	 *@param  c  The feature to be added to the ConfigElement attribute
	 ***********************************************************/
	public void addConfigElement(ConfigElement c)
	{
		configs.add(c);
	}

	/************************************************************
	 *  Description of the Method
	 ***********************************************************/
	public void uncompile()
	{
		threadGroups.clear();
		configs.clear();
	}

	/************************************************************
	 *  Adds a feature to the ThreadGroup attribute of the TestPlan object
	 *
	 *@param  group  The feature to be added to the ThreadGroup attribute
	 ***********************************************************/
	public void addThreadGroup(ThreadGroup group)
	{
		threadGroups.add(group);
	}

	/************************************************************
	 *  Description of the Method
	 *
	 *@return    Description of the Returned Value
	 ***********************************************************/
	public Collection compileTestPlan()
	{
		JMeterTreeModel tree = GuiPackage.getInstance().getTreeModel();
		tree.compileComponent((JMeterTreeNode)tree.getChild(tree.getRoot(), 0));
		return this.threadGroups;
	}

	/************************************************************
	 *  Description of the Method
	 *
	 *@param  name  Description of Parameter
	 *@return       Description of the Returned Value
	 ***********************************************************/
	public static TestPlan createTestPlan(String name)
	{
		if(plan == null)
		{
			if(name == null)
			{
				plan = new TestPlan();
			}
			else
			{
				plan = new TestPlan(name);
			}
		}
		return plan;
	}
}
