/*
 * ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2001 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Apache Software Foundation (http://www.apache.org/)."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" and
 * "Apache JMeter" must not be used to endorse or promote products
 * derived from this software without prior written permission. For
 * written permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 * "Apache JMeter", nor may "Apache" appear in their name, without
 * prior written permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */
 package org.apache.jmeter.gui.action;

import javax.swing.*;
import java.io.*;
import java.awt.event.ActionEvent;
import java.util.*;
import org.apache.jmeter.save.*;
import org.apache.jmeter.gui.*;
import org.apache.jmeter.util.*;
import org.apache.jmeter.save.xml.*;

/**
 *  Title: JMeter Description: Copyright: Copyright (c) 2000 Company: Apache
 *
 *@author     Michael Stover
 *@created    February 13, 2001
 *@version    1.0
 */

public class Save implements Command {

	private static Set commands = new HashSet();


	/**
	 *  Constructor for the Save object
	 */
	public Save() {
	}


	/**
	 *  Gets the ActionNames attribute of the Save object
	 *
	 *@return    The ActionNames value
	 */
	public Set getActionNames() {
		return commands;
	}


	/**
	 *  Description of the Method
	 *
	 *@param  e  Description of Parameter
	 */
	public void doAction(ActionEvent e) {
		JMeterComponentModel model = GuiPackage.getInstance().getCurrentModel();
		if (model instanceof Saveable) {
			JFileChooser chooser = FileDialoger.promptToSaveFile(model.getName() + ".jmx");
			FileWriter writer = null;
			try {
				writer = new FileWriter(chooser.getSelectedFile());
				TagHandler handler = (TagHandler) ((Saveable) model).getTagHandlerClass().newInstance();
				handler.startSave(writer);
				handler.save((Saveable) model, writer);
			}
			catch (Throwable ex) {
				ex.printStackTrace();
			}
			finally {
				closeWriter(writer);
				GuiPackage.getInstance().getMainFrame().repaint();
			}
		}
	}
	static {
		commands.add(JMeterUtils.getResString("save"));
	}


	/**
	 *  Description of the Method
	 *
	 *@param  writer  Description of Parameter
	 */
	private void closeWriter(FileWriter writer) {
		if (writer != null) {
			try {
				writer.close();
			}
			catch (Exception ex) {
				ex.printStackTrace();
			}
		}
	}
}
