/*
 * ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2001 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Apache Software Foundation (http://www.apache.org/)."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" and
 * "Apache JMeter" must not be used to endorse or promote products
 * derived from this software without prior written permission. For
 * written permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 * "Apache JMeter", nor may "Apache" appear in their name, without
 * prior written permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

package org.apache.jmeter.gui.tree;

import java.util.*;

import org.apache.jmeter.gui.JMeterComponentModel;

/************************************************************
 *  Title: JMeter Description: Copyright: Copyright (c) 2000 Company: Apache
 *	This class simulates a node of the tree. It eliminates the need for the swing
 *	components. The code for this class mimics JMeterTreeNode
 *	This class does not implement the JMeterComponentModel as JMeterTreeNode because
 *	we are not interested in displaying anything. Remember - No GUI here!
 *
 *	@see	org.apache.jmeter.NonGuiTree
 *
 *	@author     Tushar Bhatia
 *	@created    June 18, 2001
 *	@version    1.0
 ***********************************************************/

public class NonGuiTreeNode {
	/** All the sub nodes in this node. Each would be a NonGuiTreeNode
	*/
	private Vector children = new Vector();
	/** The user object this NonGuiTreeNode represents. This could be a ThreadGroup, SampleController,
		UrlConfig, Timer, etc.
	*/
	private JMeterComponentModel userObject = null;

	/** The parent for this Node.
		@see insertNodeInto
	*/
	private NonGuiTreeNode parent;

	/** returns an enumeration of all nodes for the current node.
	*/
	public Enumeration children(){
		return children.elements();
	}

	/** Constructor. Creates a NonGuiTreeNode based on a JMeterComponentModel
	*/
	public NonGuiTreeNode(JMeterComponentModel component){
		this.userObject = component;
	}

	/** returns the user object that this node represents.
	*/
	public JMeterComponentModel getUserObject(){
		return userObject;
	}


	public NonGuiTreeNode getParent(){
		return parent;
	}

	public void setParent(NonGuiTreeNode parent){
		this.parent = parent;
	}

	/** Inserts a child node at the specified position.
	*/
	public void insertNodeInto(NonGuiTreeNode child, int position){
		children.insertElementAt(child, position);
	}

	public int getChildCount(){
		return children.size();
	}

	public NonGuiTreeNode getChildAt(int position){
		return (NonGuiTreeNode)children.elementAt(position);
	}
}