package org.apache.jmeter.gui.util;

/**
 * Title:        Apache JMeter
 * Description:
 * Copyright:    Copyright (c) 2000
 * Company:      Apache Foundation
 * @author bo.regnlin@pc.nu (Bo Regnlin)
 * @version 1.0
 */

import java.awt.Dimension;
import java.awt.Component;

/**
 * This class is a Util for awt Component and could be
 * used to place them in center of an other.
 *
 *
 * @author  <a href="mailto:bo.regnlin@pc.nu">Bo Regnlin</a>
* @version 1.0
 */
public class ComponentUtil
{
  /**
		 * Use this static method if you want to center
	  * and set its proposion compared to the size of
	  * the current users screen size.
	  * Valid percent is between +-(0-100) minus is treated
	  * as plus, bigger than 100 is always set to 100.
		 *
		 *@param  component The component you want to center and set
size on.
		 *@param  percentOfScreen The percent of the current screensize
you want
	  *        the component to be.
		 */
  public static void centerComponentInWindow(Component component, int
percentOfScreen)
  {
	 if(percentOfScreen < 0)
	 {
		centerComponentInWindow(component, - percentOfScreen);
		return;
	 }
	 if(percentOfScreen > 100)
	 {
		centerComponentInWindow(component, 100);
		return;
	 }
	 double percent = percentOfScreen / 100.d;
	 Dimension dimension = component.getToolkit().getScreenSize();
	 component.setSize((int)(dimension.getWidth()*percent),
							 (int)(dimension.getHeight()*percent));
	 centerComponentInWindow(component);
  }

  /**
		 * Use this static method if you want to center a component in
Window.
		 *
		 *@param  component The component you want to center in window.
		 */
  public static void centerComponentInWindow(Component component)
  {
	 Dimension dimension = component.getToolkit().getScreenSize();

	 component.setLocation((int)((dimension.getWidth()-component.getWidth())/2),
								  (int)((dimension.getHeight()-component.getHeight
())/2));
	 component.validate();
	 component.repaint();
  }

  /**
		 * Use this static method if you want to center
	  * a component over an other component.
		 *
		 *@param  parent The component you want to use to place it on.
		 *@param  toBeCentered The component you want to center.
		 */
  public static void centerComponentInComponent(Component parent, Component
toBeCentered)
  {
	 toBeCentered.setLocation((int)parent.getX() + (int)((parent.getWidth()-
toBeCentered.getWidth())/2),
									 (int)parent.getY() + (int)((parent.getHeight()-
toBeCentered.getHeight())/2));

	 toBeCentered.validate();
	 toBeCentered.repaint();
  }

}
