/*
 * ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2001 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Apache Software Foundation (http://www.apache.org/)."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" and
 * "Apache JMeter" must not be used to endorse or promote products
 * derived from this software without prior written permission. For
 * written permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 * "Apache JMeter", nor may "Apache" appear in their name, without
 * prior written permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

package org.apache.jmeter.protocol.http.config;

import java.io.*;

import org.apache.jmeter.config.ConfigElement;
import org.apache.jmeter.util.JMeterUtils;

/**
 * Title:        JMeter
 * Description:
 * Copyright:    Copyright (c) 2000
 * Company:      Apache
 * @author Michael Stover
 * @version 1.0
 */

public class MultipartUrlConfig extends UrlConfig implements Serializable
{

	public static String MULTIPART_FORM = "multipart/form-data";

	private static String FILENAME = "filename";
	private static String FIELDNAME = "fieldname";
	private static String MIMETYPE = "mimetype";
	private static String BOUNDARY = "boundary";

	public MultipartUrlConfig()
	{
		super();
	}

	public MultipartUrlConfig(String boundary)
	{
		setBoundary(boundary);
	}

	public void setBoundary(String boundary)
	{
		this.putProperty(BOUNDARY,boundary);
	}

	public String getBoundary()
	{
		return (String)this.getProperty(BOUNDARY);
	}

	public String getClassLabel()
	{
		return "HTTP Multipart Request Defaults";
	}

		/**
	 *  !ToDo (Method description)
	 *
	 *@return    !ToDo (Return description)
	 */
	public Object clone()
	{
		MultipartUrlConfig newConfig = new MultipartUrlConfig();
		configureClone(newConfig);
		return newConfig;
	}

		/**
	 *  This allows config elements to combine and give a "layered" effect. for
	 *  example, say there are two HTTPConfigElements, which have properties for
	 *  domain, path, method, and parameters. If element A has everything filled
	 *  in, but null for domain, and element B is added, which has only domain
	 *  filled in, then after adding B to A, A will have the domain from B. If A
	 *  already had a domain, then the correct behavior is for A to ignore the
	 *  addition of element B.
	 *
	 *@param  config  !ToDo
	 */
	public void addConfigElement(ConfigElement config)
	{
		if (config instanceof UrlConfig)
		{
			updatePropertyIfAbsent((UrlConfig)config);
		}
	}

	public Class getGuiClass()
	{
		return org.apache.jmeter.protocol.http.config.gui.MultipartUrlConfigGui.class;
	}

	public void setFilename(String filename)
	{
		this.putProperty(FILENAME,filename);
	}

	public String getFilename()
	{
		return (String)this.getProperty(FILENAME);
	}

	public void setFileFieldName(String name)
	{
		this.putProperty(FIELDNAME,name);
	}

	public String getFileFieldName()
	{
		return (String)this.getProperty(FIELDNAME);
	}

	public void setMimeType(String type)
	{
		this.putProperty(MIMETYPE,type);
	}

	public String getMimeType()
	{
		return (String)this.getProperty(MIMETYPE);
	}

	public InputStream getFileStream() throws IOException
	{
		return new BufferedInputStream(new FileInputStream(getFilename()));
	}

		/**
	 * This method allows a proxy server to send over the raw text from a browser's
	 * output stream to be parsed and stored correctly into the UrlConfig object.
	 */
	public void parseArguments(String queryString)
	{
		String[] parts = JMeterUtils.split(queryString,"--"+getBoundary());
		for (int i = 0; i < parts.length; i++)
		{
			if(parts[i].indexOf("filename=") > -1)
			{
				int index = parts[i].indexOf("name=\"")+6;
				String name = parts[i].substring(index,parts[i].indexOf("\"",index));
				index = parts[i].indexOf("filename=\"")+10;
				String filename = parts[i].substring(index,parts[i].indexOf("\"",index));
				index = parts[i].indexOf("\n",index);
				index = parts[i].indexOf(":",index)+1;
				String mimetype = parts[i].substring(index,parts[i].indexOf("\n",index)).trim();
				this.setFileFieldName(name);
				this.setFilename(filename);
				this.setMimeType(mimetype);
			}
			else if(parts[i].indexOf("name=") > -1)
			{
				int index = parts[i].indexOf("name=\"")+6;
				String name = parts[i].substring(index,parts[i].indexOf("\"",index));
				index = parts[i].indexOf("\n",index)+2;
				String value = parts[i].substring(index).trim();
				this.addArgument(name,value);
			}
		}
	}
}
