/*
 * ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2001 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Apache Software Foundation (http://www.apache.org/)."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" and
 * "Apache JMeter" must not be used to endorse or promote products
 * derived from this software without prior written permission. For
 * written permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 * "Apache JMeter", nor may "Apache" appear in their name, without
 * prior written permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */
package org.apache.jmeter.protocol.http.gui;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.table.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.text.*;
import org.apache.jmeter.protocol.http.control.*;
import org.apache.jmeter.util.JMeterUtils;
import org.apache.jmeter.gui.*;

/**
 *  Allows the user to specify if she needs cookie services, and give parameters
 *  for this service.
 *
 *@author     $Author: khammond $
 *@created    $Date: 2001/11/22 16:48:27 $
 *@version    $Revision: 1.10 $
 */
public class CookiePanel extends JPanel implements ModelSupported, ActionListener
{
	CookieManager manager;
	InnerTableModel tableModel;
	JTable cookieTable;
	JTextField nicknameField;
	JButton addButton;
	JButton deleteButton;
	JButton loadButton;
	JButton saveButton;
	JPanel cookieManagerPanel;
	JTextPane nicknameTextPane;
	JTextArea nicknameText;
	NamePanel namePanel;

	/**
	 *  Default constructor
	 */
	public CookiePanel()
	{
	}

	/**
	 *  !ToDo (Method description)
	 *
	 *@param  model  !ToDo (Parameter description)
	 */
	public void setModel(Object model)
	{
		manager = (CookieManager) model;
		tableModel = new InnerTableModel(manager);
		init();
	}

	/**
	 *  Gets a CookieManager to manage the file that is currently selected. Null is
	 *  returned if no file is currently selected. Null will also be returned if
	 *  there is a problem reading the file while getting the CookieManager.
	 *
	 *@return     !ToDo (Return description)
	 *@returns    A CookieManager for the current file, or null
	 */
	public CookieManager getCookieMgr()
	{
		return manager;
	}

	/**
	 *  Description of the Method
	 */
	public void updateGui()
	{
		tableModel.fireTableDataChanged();
	}

	/************************************************************
	 *  Description of the Method
	 *
	 *@param  e  Description of Parameter
	 ***********************************************************/
	public void actionPerformed(ActionEvent e)
	{
		String action = e.getActionCommand();

		if (action.equals("Delete"))
		{
			if (tableModel.getRowCount() > 0)
			{
				// If a table cell is being edited, we must cancel the editing before 
				// deleting the row
				if (cookieTable.isEditing())
				{
					TableCellEditor cellEditor = cookieTable.getCellEditor(cookieTable.getEditingRow(), cookieTable.getEditingColumn());
					cellEditor.cancelCellEditing();
				}

				int rowSelected = cookieTable.getSelectedRow();

				if (rowSelected != -1)
				{
					tableModel.removeRow(rowSelected);
					tableModel.fireTableDataChanged();

					// Disable the DELETE and SAVE buttons if no rows remaining after delete
					if (tableModel.getRowCount() == 0)
					{
						deleteButton.setEnabled(false);
						saveButton.setEnabled(false);
					}
				
					// Table still contains one or more rows, so highlight (select)
					// the appropriate one. 
					else
					{
						int rowToSelect = rowSelected;

						if (rowSelected >= tableModel.getRowCount())
						{
							rowToSelect = rowSelected - 1;
						}

						cookieTable.setRowSelectionInterval(rowToSelect, rowToSelect);
					}
				}
			}
		}
		else if(action.equals("Add"))
		{
			// If a table cell is being edited, we should accept the current value
			// and stop the editing before adding a new row.
			if (cookieTable.isEditing())
			{
				TableCellEditor cellEditor = cookieTable.getCellEditor(cookieTable.getEditingRow(), cookieTable.getEditingColumn());
				cellEditor.stopCellEditing();
			}

			tableModel.addNewRow();
			tableModel.fireTableDataChanged();

			// Enable the DELETE and SAVE buttons if they are currently disabled.
			if (!deleteButton.isEnabled())
			{
				deleteButton.setEnabled(true);
			}
			if (!saveButton.isEnabled())
			{
				saveButton.setEnabled(true);
			}

			// Highlight (select) the appropriate row.
			int rowToSelect = tableModel.getRowCount() - 1;
			cookieTable.setRowSelectionInterval(rowToSelect, rowToSelect);
		}
		else if (action.equals("Load"))
		{
			try
			{
				File tmp = FileDialoger.promptToOpenFile().getSelectedFile();
				if (tmp != null)
				{
					manager.addFile(tmp.getAbsolutePath());
					tableModel.fireTableDataChanged();

					if (tableModel.getRowCount() > 0)
					{
						deleteButton.setEnabled(true);
						saveButton.setEnabled(true);
					}
				}
			}
			catch (IOException ex)
			{
				ex.printStackTrace();
			}
		}
		else if (action.equals("Save"))
		{
			try
			{
				File tmp = FileDialoger.promptToSaveFile(null).getSelectedFile();
				if (tmp != null)
				{
					manager.save(tmp.getAbsolutePath());
				}
			}
			catch (IOException ex)
			{
				ex.printStackTrace();
			}
		}
	}

	/**
	 *  Shows the main cookie configuration panel
	 */
	public void init()
	{
		// set the layout of the control panel
		this.setLayout(new VerticalLayout(5, VerticalLayout.LEFT));

		cookieManagerPanel = new JPanel();

		Border margin = new EmptyBorder(10, 10, 5, 10);
		cookieManagerPanel.setBorder(margin);

		cookieManagerPanel.setLayout(new VerticalLayout(5, VerticalLayout.LEFT));

		JLabel panelTitleLabel = new JLabel(JMeterUtils.getResString("cookie_manager_title"));
		Font curFont = panelTitleLabel.getFont();
		int curFontSize = curFont.getSize();
		curFontSize += 4;
		panelTitleLabel.setFont(new Font(curFont.getFontName(), curFont.getStyle(), curFontSize));
		cookieManagerPanel.add(panelTitleLabel);

		namePanel = new NamePanel(manager);
		cookieManagerPanel.add(namePanel);

		JPanel cookieTablePanel = createCookieTablePanel();
		cookieManagerPanel.add(cookieTablePanel);

		this.add(cookieManagerPanel);
	}

	public JPanel createCookieTablePanel()
	{
		Border margin = new EmptyBorder(5, 10, 10, 10);

		JPanel tempPanel = new JPanel();
		tempPanel.setLayout(new VerticalLayout(0, VerticalLayout.CENTER));
		tempPanel.setBorder(new CompoundBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), JMeterUtils.getResString("cookies_stored")), margin));

		// create the JTable that holds one cookie per row
		cookieTable = new JTable(tableModel);
		cookieTable.setCellSelectionEnabled(true);
		cookieTable.setRowSelectionAllowed(true);
		cookieTable.setColumnSelectionAllowed(false);
		cookieTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

		// create a JScrollPane and place the cookie JTable inside it
		JScrollPane scroller = new JScrollPane(cookieTable);
		cookieTable.setPreferredScrollableViewportSize(new Dimension(520, 150));
		JTableHeader tableHeader = cookieTable.getTableHeader();
		scroller.setColumnHeaderView(tableHeader);

		tempPanel.add(scroller);

		// ADD button
		addButton = new JButton(JMeterUtils.getResString("add"));
		addButton.setMnemonic('A');
		addButton.setActionCommand("Add");
		addButton.addActionListener(this);

		// DELETE button
		deleteButton = new JButton(JMeterUtils.getResString("delete"));

		if (tableModel.getRowCount() == 0)
		{
			deleteButton.setEnabled(false);
		}
		else
		{
			deleteButton.setEnabled(true);
		}

		deleteButton.setMnemonic('D');
		deleteButton.setActionCommand("Delete");
		deleteButton.addActionListener(this);

		// LOAD button
		loadButton = new JButton(JMeterUtils.getResString("load"));
		loadButton.setMnemonic('L');
		loadButton.setActionCommand("Load");
		loadButton.addActionListener(this);

		// SAVE button
		saveButton = new JButton(JMeterUtils.getResString("save"));

		if (tableModel.getRowCount() == 0)
		{
			saveButton.setEnabled(false);
		}
		else
		{
			saveButton.setEnabled(true);
		}

		saveButton.setMnemonic('S');
		saveButton.setActionCommand("Save");
		saveButton.addActionListener(this);

		// Button Panel
		JPanel buttonPanel = new JPanel();
		buttonPanel.add(addButton);
		buttonPanel.add(deleteButton);
		buttonPanel.add(loadButton);
		buttonPanel.add(saveButton);

		tempPanel.add(Box.createRigidArea(new Dimension(0, 10)));
		tempPanel.add(buttonPanel);

		return tempPanel;
	}

	/**
	 *  Description of the Class
	 *
	 *@author     mike
	 *@created    July 14, 2001
	 */
	public class InnerTableModel extends AbstractTableModel
	{
		CookieManager manager;

		/**
		 *  Constructor for the InnerTableModel object
		 *
		 *@param  man  Description of Parameter
		 */
		public InnerTableModel(CookieManager man)
		{
			manager = man;
		}

		/************************************************************
		 *  Description of the Method
		 *
		 *@param  row  Description of Parameter
		 ***********************************************************/
		public void removeRow(int row)
		{
			manager.remove(row);
		}

		/**
		 *  
		 */
		public void addNewRow()
		{
			manager.add();
		}

		/**
		 *  required by table model interface
		 *
		 *@return    The RowCount value
		 */
		public int getRowCount()
		{
			//System.out.println("manager.getCookieCount(): " + manager.getCookieCount());
			return manager.getCookieCount();
		}

		/**
		 *  required by table model interface
		 *
		 *@return    The ColumnCount value
		 */
		public int getColumnCount()
		{
			return manager.getColumnCount();
		}

		/**
		 *  required by table model interface
		 *
		 *@param  column  Description of Parameter
		 *@return         The ColumnName value
		 */
		public String getColumnName(int column)
		{
			return manager.getColumnName(column);
		}


		public boolean isCellEditable(int row, int column)
		{
			// all table cells are editable
			return true;
		}

		public Class getColumnClass(int column) 
		{
			return getValueAt(0, column).getClass();
		}

		/**
		 *  required by table model interface
		 *
		 *@param  row     Description of Parameter
		 *@param  column  Description of Parameter
		 *@return         The ValueAt value
		 */
		public Object getValueAt(int row, int column)
		{
			Cookie cook = manager.getCookie(row);
			if (column == 0)
			{
				return cook.getName();
			}
			else if (column == 1)
			{
				return cook.getValue();
			}
			else if (column == 2)
			{
				return cook.getDomain();
			}
			else if (column == 3)
			{
				return cook.getPath();
			}
			else if (column == 4)
			{
				return new Boolean(cook.getSecure());
			}
			else if (column == 5)
			{
				return manager.convertLongToDateFormatStr(cook.getExpires());
			}
			return null;
		}

		/************************************************************
		 *  Sets the ValueAt attribute of the Arguments object
		 *
		 *@param  value  The new ValueAt value
		 *@param  row    The new ValueAt value
		 *@param  col    The new ValueAt value
		 ***********************************************************/
		public void setValueAt(Object value, int row, int column)
		{
			Cookie cook = manager.getCookie(row);

			if (column == 0)
			{
				cook.setName((String)value);
			}
			else if (column == 1)
			{
				cook.setValue((String)value);
			}
			else if (column == 2)
			{
				cook.setDomain((String)value);
			}
			else if (column == 3)
			{
				cook.setPath((String)value);
			}
			else if (column == 4)
			{
				cook.setSecure(((Boolean)value).booleanValue());
			}
			else if (column == 5)
			{
				cook.setExpires(manager.convertDateFormatStrToLong((String)value));
			}
		}
	}
}
