/*
 * ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2001 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Apache Software Foundation (http://www.apache.org/)."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" and
 * "Apache JMeter" must not be used to endorse or promote products
 * derived from this software without prior written permission. For
 * written permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 * "Apache JMeter", nor may "Apache" appear in their name, without
 * prior written permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */
package org.apache.jmeter.protocol.jdbc.config.gui;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;

import org.apache.jmeter.gui.*;
import org.apache.jmeter.util.JMeterUtils;
import org.apache.jmeter.config.gui.LoginConfigGui;
import org.apache.jmeter.protocol.jdbc.config.DbConfig;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author Michael Stover
 * @version 1.0
 */

public class DbConfigGui extends JPanel implements ModelSupported,KeyListener
{
	private static String PROTOCOL = "protocol";
	private static String URL = "url";
	private static String DRIVER = "driver";

	private DbConfig model;
	private boolean displayName;
	private NamePanel namePanel;
	private LoginConfigGui loginGui;
	private JTextField urlField = new JTextField(20);
	private JTextField driverField = new JTextField(20);

	public DbConfigGui()
	{
		displayName = true;
	}

	public DbConfigGui(boolean displayName)
	{
		this.displayName = displayName;
	}

	public void setModel(Object model)
	{
		this.model = (DbConfig)model;
		init();
	}

	public void updateGui()
	{
		driverField.setText(model.getDriver());
		urlField.setText(model.getUrl());
	}

	private void init()
	{
		if (displayName)
		{
			this.setLayout(new VerticalLayout(5, VerticalLayout.LEFT, VerticalLayout.TOP));

			// MAIN PANEL
			JPanel mainPanel = new JPanel();
			Border margin = new EmptyBorder(10, 10, 5, 10);
			mainPanel.setBorder(margin);
			mainPanel.setLayout(new VerticalLayout(5, VerticalLayout.LEFT));

			// TITLE
			JLabel panelTitleLabel = new JLabel(JMeterUtils.getResString("database_login_title"));
			Font curFont = panelTitleLabel.getFont();
			int curFontSize = curFont.getSize();
			curFontSize += 4;
			panelTitleLabel.setFont(new Font(curFont.getFontName(), curFont.getStyle(), curFontSize));
			mainPanel.add(panelTitleLabel);

			// NAME
			namePanel = new NamePanel(model);
			mainPanel.add(namePanel);

			// URL and JDBC PROPS
			JPanel urlJDBCPanel = new JPanel();
			urlJDBCPanel.setLayout(new VerticalLayout(5,VerticalLayout.LEFT));
			urlJDBCPanel.setBorder(BorderFactory.createTitledBorder(JMeterUtils.getResString("database_url_jdbc_props")));
			
			// URL
			urlJDBCPanel.add(getUrlPanel());

			// DRIVER
			urlJDBCPanel.add(getDriverPanel());

			mainPanel.add(urlJDBCPanel);

			// LOGIN
			loginGui = new LoginConfigGui(false);
			loginGui.setModel(model);
			JPanel loginPanel = new JPanel();
			loginPanel.setBorder(BorderFactory.createTitledBorder(JMeterUtils.getResString("login_config")));
			loginPanel.add(loginGui);
			mainPanel.add(loginPanel);

			this.add(mainPanel);
		}
		else
		{
			this.setLayout(new VerticalLayout(5,VerticalLayout.LEFT));

			// URL and JDBC PROPS
			JPanel urlJDBCPanel = new JPanel();
			urlJDBCPanel.setLayout(new VerticalLayout(5,VerticalLayout.LEFT));
			urlJDBCPanel.setBorder(BorderFactory.createTitledBorder(JMeterUtils.getResString("database_url_jdbc_props")));
			
			// URL
			urlJDBCPanel.add(getUrlPanel());

			// DRIVER
			urlJDBCPanel.add(getDriverPanel());

			this.add(urlJDBCPanel);

			// LOGIN
			loginGui = new LoginConfigGui(false);
			loginGui.setModel(model);
			JPanel loginPanel = new JPanel();
			loginPanel.setBorder(BorderFactory.createTitledBorder(JMeterUtils.getResString("login_config")));
			loginPanel.add(loginGui);
			this.add(loginPanel);
		}
	}

	private JPanel getDriverPanel()
	{
		JPanel panel = new JPanel();
		panel.add(new JLabel(JMeterUtils.getResString("database_driver_class")));
		driverField.setText(model.getDriver());
		driverField.setName(DRIVER);
		driverField.addKeyListener(this);
		panel.add(driverField);
		return panel;
	}

	private JPanel getUrlPanel()
	{
		JPanel panel = new JPanel();
		panel.add(new JLabel(JMeterUtils.getResString("database_url")));
		urlField.setText(model.getUrl());
		urlField.setName(URL);
		urlField.addKeyListener(this);
		panel.add(urlField);
		return panel;
	}

	public void keyPressed(KeyEvent e)
	{
	}

	public void keyTyped(KeyEvent e)
	{
	}

	public void keyReleased(KeyEvent e)
	{
		String name = e.getComponent().getName();
		if(name.equals(DRIVER))
		{
			model.setDriver(driverField.getText());
		}
		else if(name.equals(URL))
		{
			model.setUrl(urlField.getText());
		}
	}
}
