/*
 * ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2001 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Apache Software Foundation (http://www.apache.org/)."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" and
 * "Apache JMeter" must not be used to endorse or promote products
 * derived from this software without prior written permission. For
 * written permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 * "Apache JMeter", nor may "Apache" appear in their name, without
 * prior written permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */
 package org.apache.jmeter.protocol.jdbc.config.gui;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;

import org.apache.jmeter.gui.*;
import org.apache.jmeter.util.JMeterUtils;
import org.apache.jmeter.protocol.jdbc.config.*;

/**
 * Title:        JMeter
 * Description:
 * Copyright:    Copyright (c) 2000
 * Company:      Apache
 * @author Michael Stover
 * @version 1.0
 */

public class PoolConfigGui extends JPanel implements ModelSupported,KeyListener
{
	private static String CONNECTIONS = "connections";
	private static String MAXUSE = "maxuse";

	private PoolConfig model;
	private boolean displayName;

	private NamePanel namePanel;
	private JTextField connField = new JTextField(5);
	private JTextField maxUseField = new JTextField(5);

	public PoolConfigGui()
	{
		displayName = true;
	}

	public PoolConfigGui(boolean displayName)
	{
		this.displayName = displayName;
	}

	public void updateGui()
	{
		connField.setText(Integer.toString(model.getNumConnections()));
		maxUseField.setText(Integer.toString(model.getMaxUse()));
	}

	public void setModel(Object model)
	{
		this.model = (PoolConfig)model;
		init();
	}

	private void init()
	{
		if (displayName)
		{
			this.setLayout(new VerticalLayout(5, VerticalLayout.LEFT, VerticalLayout.TOP));

			// MAIN PANEL
			JPanel mainPanel = new JPanel();
			Border margin = new EmptyBorder(10, 10, 5, 10);
			mainPanel.setBorder(margin);
			mainPanel.setLayout(new VerticalLayout(5, VerticalLayout.LEFT));

			// TITLE
			JLabel panelTitleLabel = new JLabel(JMeterUtils.getResString("database_conn_pool_title"));
			Font curFont = panelTitleLabel.getFont();
			int curFontSize = curFont.getSize();
			curFontSize += 4;
			panelTitleLabel.setFont(new Font(curFont.getFontName(), curFont.getStyle(), curFontSize));
			mainPanel.add(panelTitleLabel);

			// NAME
			namePanel = new NamePanel(model);
			mainPanel.add(namePanel);

			// CONNECTION POOL
			JPanel connPoolPanel = new JPanel();
			connPoolPanel.setLayout(new VerticalLayout(5,VerticalLayout.LEFT));
			connPoolPanel.setBorder(BorderFactory.createTitledBorder(JMeterUtils.getResString("database_conn_pool_props")));

			// NUMBER OF CONNECTIONS
			connPoolPanel.add(createConnPanel());

			// MAX USAGE
			connPoolPanel.add(createMaxUsePanel());

			mainPanel.add(connPoolPanel);

			this.add(mainPanel);
		}
		else
		{
			this.setLayout(new VerticalLayout(5,VerticalLayout.LEFT));

			// CONNECTION POOL
			JPanel connPoolPanel = new JPanel();
			connPoolPanel.setLayout(new VerticalLayout(5,VerticalLayout.LEFT));
			connPoolPanel.setBorder(BorderFactory.createTitledBorder(JMeterUtils.getResString("database_conn_pool_props")));

			// NUMBER OF CONNECTIONS
			connPoolPanel.add(createConnPanel());

			// MAX USAGE
			connPoolPanel.add(createMaxUsePanel());

			this.add(connPoolPanel);
		}
	}

	private JPanel createConnPanel()
	{
		JPanel panel = new JPanel();
		panel.add(new JLabel(JMeterUtils.getResString("database_conn_pool_size")));
		connField.setText(Integer.toString(model.getNumConnections()));
		connField.setName(CONNECTIONS);
		connField.addKeyListener(this);
		panel.add(connField);
		return panel;
	}

	private JPanel createMaxUsePanel()
	{
		JPanel panel = new JPanel();
		panel.add(new JLabel(JMeterUtils.getResString("database_conn_pool_max_usage")));
		maxUseField.setText(Integer.toString(model.getMaxUse()));
		maxUseField.setName(MAXUSE);
		maxUseField.addKeyListener(this);
		panel.add(maxUseField);
		return panel;
	}

	public void keyPressed(KeyEvent e)
	{
	}

	public void keyTyped(KeyEvent e)
	{
	}

	public void keyReleased(KeyEvent e)
	{
		String name = e.getComponent().getName();

		if (name.equals(CONNECTIONS))
		{
			try
			{
				model.setNumConnections(Integer.parseInt(connField.getText()));
			}
			catch (NumberFormatException nfe)
			{
				if (connField.getText().length() > 0)
				{
					JOptionPane.showMessageDialog(this, "You must enter a valid number",
							"Invalid data", JOptionPane.WARNING_MESSAGE);

					// Right now, the cleanest thing to do is simply clear the 
					// entire text field. We do not want to set the text to
					// the default because that would be confusing to the user.
					// For example, the user typed "5t" instead of "56". After
					// the user closes the error dialog, the text would change
					// from "5t" to "1".  A litle confusing. If anything, it
					// should display just "5". Future enhancement...
					connField.setText("");
				}

				model.setNumConnections(model.getDefaultNumConnections());
			}
		}
		else if (name.equals(MAXUSE))
		{
			try
			{
				model.setMaxUse(Integer.parseInt(maxUseField.getText()));
			}
			catch (NumberFormatException nfe)
			{
				if (maxUseField.getText().length() > 0)
				{
					JOptionPane.showMessageDialog(this, "You must enter a valid number",
							"Invalid data", JOptionPane.WARNING_MESSAGE);

					// Right now, the cleanest thing to do is simply clear the 
					// entire text field. We do not want to set the text to
					// the default because that would be confusing to the user.
					// For example, the user typed "5t" instead of "56". After
					// the user closes the error dialog, the text would change
					// from "5t" to "1".  A litle confusing. If anything, it
					// should display just "5". Future enhancement...
					maxUseField.setText("");
				}

				model.setMaxUse(model.getDefaultMaxUse());
			}
		}
	}
}
