/*
 * ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2001 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Apache Software Foundation (http://www.apache.org/)."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" and
 * "Apache JMeter" must not be used to endorse or promote products
 * derived from this software without prior written permission. For
 * written permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 * "Apache JMeter", nor may "Apache" appear in their name, without
 * prior written permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */
package org.apache.jmeter.reporters;

import org.apache.jmeter.gui.*;
import java.util.*;
import java.lang.ref.*;
import java.io.*;
import org.apache.jmeter.samplers.*;
import org.apache.jmeter.visualizers.ViewResultsVisualizer;
import org.apache.jmeter.save.Saveable;
import org.apache.jmeter.util.JMeterUtils;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author Michael Stover
 * @version 1.0
 */

public class ResultCollector implements JMeterComponentModel,SampleListener,Saveable,Clearable,
		  Serializable
{

	 protected List results = Collections.synchronizedList(new ArrayList());
	 private String name;
	 private ModelSupported listener;
	 private int current;

	 public ResultCollector()
	 {
		  current = -1;
	 }

	 public Class getTagHandlerClass()
	 {
		  return org.apache.jmeter.save.handlers.JMeterComponentHandler.class;
	 }

	 public void setListener(ModelSupported listener)
	 {
		  this.listener = listener;
	 }

	 public void addJMeterComponent(JMeterComponentModel child)
	 {
	 }

	 public Class getGuiClass()
	 {
		  return org.apache.jmeter.visualizers.ViewResultsVisualizer.class;
	 }

	 public String getName()
	 {
		  return name;
	 }

	 public void setName(String name)
	 {
		  this.name = name;
	 }

	 public Collection getAddList()
	 {
		  return null;
	 }

	 public String getClassLabel()
	 {
                  return JMeterUtils.getResString("view_results_title");
	 }

	 public void uncompile()
	 {
		  this.clear();
	 }

	 public void clear()
	 {
		  results.clear();
		  current = -1;
		  if(listener != null && listener instanceof Clearable)
		  {
				((Clearable)listener).clear();
		  }
	 }

	 public String getSampleLabel()
	 {
		try
		{
		  return (String)getCurrentSample().getValue(SampleResult.SAMPLE_LABEL);
		}
		catch(NullPointerException e)
		{
			return "";
		}
	 }

	 public SampleResult getCurrentSample()
	 {
		  if(current < results.size() && current >= 0)
		  {
				SampleResult res = (SampleResult)results.get(current);
				return res;
		  }
		  else if(results.size() > 0)
		  {
			return (SampleResult)results.get(results.size()-1);
		  }
		  else
		  {
			return null;
		  }
	 }

	 public String getTextResponse()
	 {
		try
		{
		  return new String((byte[])getCurrentSample().getValue(SampleResult.TEXT_RESPONSE));
		}
		catch(NullPointerException e)
		{
			return "";
		}
	 }

	 public boolean next()
	 {
		  if(current < results.size()-1)
		  {
				current++;
				return true;
		  }
		  else
		  {
			return false;
		  }
	 }

	 public void sampleStarted(SampleEvent e)
	 {
	 }

	 public void sampleStopped(SampleEvent e)
	 {
	 }

	 public void sampleOccurred(SampleEvent e)
	 {
		  results.add(e.getResult());
		  if(results.size() > 100)
		  {
			results.remove(0);
			if(current > 0)
			{
				current--;
			}
		  }
		  if(listener != null)
		  {
				listener.updateGui();
		  }
	 }
}
