/*
 * ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2001 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Apache Software Foundation (http://www.apache.org/)."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" and
 * "Apache JMeter" must not be used to endorse or promote products
 * derived from this software without prior written permission. For
 * written permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 * "Apache JMeter", nor may "Apache" appear in their name, without
 * prior written permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */
package org.apache.jmeter.samplers;

import java.util.*;
import java.io.Serializable;

import org.apache.log4j.*;

/************************************************************
 *  This is a nice packaging for the various information returned from taking a
 *  sample of an entry.
 *
 *@author     $Author: mstover1 $
 *@created    $Date: 2002/02/16 03:21:42 $
 *@version    $Revision: 1.15 $
 ***********************************************************/
public class SampleResult implements Serializable
{
	//public final static String URL = "sampler.url";

	Map map;
	long time;
	public final static String DISPLAY_NAME = "sr.display_name";
	public final static String ASSERTION_RESULTS = "sr.assertion_results";
	public final static String TEXT_RESPONSE = "sampler.RESPONSE";
	public final static String SAMPLE_LABEL = "sampler.LABEL";
	public final static String SUCCESS = "sampler.SUCCESS";
	public final static String RESULT_LIST = "sample.RESULT_LIST";
	public final static String TOTAL_TIME = "sample.TOTAL_TIME";

	private transient static Category catClass =
		Category.getInstance(SampleResult.class.getName());


	/************************************************************
	 *  !ToDo (Constructor description)
	 ***********************************************************/
	public SampleResult()
	{
		map = new HashMap();
		time = 0;
		putValue(DISPLAY_NAME, "Display name of SampleResult not set");
	}

	/************************************************************
	 *  Set the time this sample took to occur.
	 *
	 *@param  t  !ToDo (Parameter description)
	 ***********************************************************/
	public void setTime(long t)
	{
		time = t;
	}

	/************************************************************
	 *  Get the time it took this sample to occur.
	 *
	 *@return    !ToDo (Return description)
	 ***********************************************************/
	public long getTime()
	{
		return time;
	}

	/************************************************************
	 *  !ToDoo (Method description)
	 *
	 *@return    !ToDo (Return description)
	 ***********************************************************/
	public boolean isSuccessfull()
	{
		Boolean success = (Boolean)getValue(SUCCESS);
		if (success != null)
		{
			return success.booleanValue();
		}
		else
		{
			return false;
		}
	}

	/************************************************************
	 *  All keys in this interface must be Strings.
	 *
	 *@param  key  !ToDo (Parameter description)
	 *@return      !ToDo (Return description)
	 ***********************************************************/
	public Object getValue(String key)
	{
		Object value = map.get(key);
		if(catClass.isDebugEnabled())
		{
			catClass.debug("getValue1 : key - " + key + " ; value - " + value);
		}
		return value;
	}

	/************************************************************
	 *  Get an array of all the names
	 *
	 *@return    !ToDo (Return description)
	 ***********************************************************/
	public String[] getNames()
	{
		return (String[])map.keySet().toArray(new String[0]);
	}

	/************************************************************
	 *  !ToDo (Method description)
	 *
	 *@param  key    !ToDo (Parameter description)
	 *@param  value  !ToDo (Parameter description)
	 ***********************************************************/
	public void putValue(String key, Object value)
	{
		map.put(key, value);
	}

	/************************************************************
	 *  Get a ready made iterator to go through the names
	 *
	 *@return    !ToDo (Return description)
	 ***********************************************************/
	public java.util.Iterator nameIterator()
	{
		return map.keySet().iterator();
	}

	/************************************************************
	 *  Remove a key-value pair.
	 *
	 *@param  key  !ToDo (Parameter description)
	 ***********************************************************/
	public void remove(String key)
	{
		map.remove(key);
	}

	/************************************************************
	 *  Generates an iterator for all the values stored under one key.
	 *
	 *@param  key  !ToDo (Parameter description)
	 *@return      !ToDo (Return description)
	 ***********************************************************/
	public java.util.Iterator valueIterator(String key)
	{
		Object value = null;
		Object[] values = null;
		try
		{
			values = (Object[])map.get(key);
		}
		catch (Exception e)
		{
			value = map.get(key);
		}
		if (value == null)
		{
			if (values == null)
			{
				List list = new LinkedList();
				return list.iterator();
			}
			else
			{
				List list = new LinkedList();
				for (int x = 0; x < values.length; list.add(values[x++]))
				{
					;
				}
				return list.iterator();
			}
		}
		else if (value instanceof Collection)
		{
			Collection col = (Collection)value;
			return col.iterator();
		}
		else
		{
			List list = new LinkedList();
			list.add(value);
			return list.iterator();
		}
	}

	/************************************************************
	 *  Get the value for the key as a String object
	 *
	 *@param  key  !ToDo (Parameter description)
	 *@return      !ToDo (Return description)
	 ***********************************************************/
	public String toString(String key)
	{
		Object value = map.get(key);
		if (value == null)
		{
			value = new String("");
		}
		return value.toString();
	}

	/**
	 * Returns the display name
	 *
	 * @return	display name of this sample result
	 */
	public String toString()
	{
		String toString = (String)getValue(DISPLAY_NAME);
		return toString;
	}
}
