/*
 * ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2001 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Apache Software Foundation (http://www.apache.org/)."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" and
 * "Apache JMeter" must not be used to endorse or promote products
 * derived from this software without prior written permission. For
 * written permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 * "Apache JMeter", nor may "Apache" appear in their name, without
 * prior written permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

package org.apache.jmeter.timers;

import java.util.*;
import java.io.*;

import org.apache.jmeter.gui.popup.*;
import org.apache.jmeter.gui.JMeterComponentModel;
import org.apache.jmeter.save.Saveable;
import org.apache.jmeter.util.JMeterUtils;

/************************************************************
 *  This class implements a constant timer with its own panel and fields for
 *  value update and user interaction.
 *
 *@author     <a href="mailto:stefano@apache.org">Stefano Mazzocchi</a>
 *@created    $Date: 2002/03/04 23:08:55 $
 *@version    $Revision: 1.18 $ $Date: 2002/03/04 23:08:55 $
 ***********************************************************/

public class ConstantTimer implements Timer, JMeterComponentModel, Saveable,
		Serializable
{
	private final long DEFAULT_DELAY = 300;
	private long delay = DEFAULT_DELAY;
	private String name;
	private static List addableList = new LinkedList();

	/************************************************************
	 *  !ToDo (Constructor description)
	 ***********************************************************/
	public ConstantTimer()
	{
	}

	public void addJMeterComponent(JMeterComponentModel child)
	{
	}

	/************************************************************
	 *  !ToDo (Method description)
	 *
	 *@param  name  !ToDo (Parameter description)
	 ***********************************************************/
	public void setName(String name)
	{
		this.name = name;
	}

	/************************************************************
	 *  !ToDo (Method description)
	 *
	 *@param  delay  !ToDo (Parameter description)
	 ***********************************************************/
	public void setDelay(long delay)
	{
		this.delay = delay;
	}

	/************************************************************
	 *  !ToDo (Method description)
	 *
	 *@param  range  !ToDo (Parameter description)
	 ***********************************************************/
	public void setRange(double range)
	{
	}

	/************************************************************
	 *  !ToDoo (Method description)
	 *
	 *@return    !ToDo (Return description)
	 ***********************************************************/
	public long getDelay()
	{
		return delay;
	}

	/************************************************************
	 *  !ToDoo (Method description)
	 *
	 *@return    !ToDo (Return description)
	 ***********************************************************/
	public long getDefaultDelay()
	{
		return DEFAULT_DELAY;
	}

	/************************************************************
	 *  !ToDoo (Method description)
	 *
	 *@return    !ToDo (Return description)
	 ***********************************************************/
	public Class getTagHandlerClass()
	{
		return org.apache.jmeter.save.handlers.TimerHandler.class;
	}

	/************************************************************
	 *  !ToDoo (Method description)
	 *
	 *@return    !ToDo (Return description)
	 ***********************************************************/
	public String getName()
	{
		return name;
	}

	/************************************************************
	 *  !ToDoo (Method description)
	 *
	 *@return    !ToDo (Return description)
	 ***********************************************************/
	public Collection getAddList()
	{
		return addableList;
	}

	/************************************************************
	 *  !ToDoo (Method description)
	 *
	 *@return    !ToDo (Return description)
	 ***********************************************************/
	public boolean isEditable()
	{
		return true;
	}

	/************************************************************
	 *  !ToDoo (Method description)
	 *
	 *@return    !ToDo (Return description)
	 ***********************************************************/
	public Class getGuiClass()
	{
		return org.apache.jmeter.timers.gui.ConstantTimerGui.class;
	}

	/************************************************************
	 *  !ToDoo (Method description)
	 *
	 *@return    !ToDo (Return description)
	 ***********************************************************/
	public double getRange()
	{
		return (double)0;
	}

	/************************************************************
	 *  !ToDoo (Method description)
	 *
	 *@return    !ToDo (Return description)
	 ***********************************************************/
	public String getClassLabel()
	{
		return JMeterUtils.getResString("constant_timer_title");
	}

	/************************************************************
	 *  !ToDo (Method description)
	 *
	 *@return    !ToDo (Return description)
	 ***********************************************************/
	public long delay()
	{
		return this.delay;
	}

	/************************************************************
	 *  !ToDo (Method description)
	 ***********************************************************/
	public void uncompile()
	{
	}

	/************************************************************
	 *  !ToDo (Method description)
	 *
	 *@return    !ToDo (Return description)
	 ***********************************************************/
	public String toString()
	{
		return JMeterUtils.getResString("constant_timer_memo");
	}
}
