/*
 * ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2001 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Apache Software Foundation (http://www.apache.org/)."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" and
 * "Apache JMeter" must not be used to endorse or promote products
 * derived from this software without prior written permission. For
 * written permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 * "Apache JMeter", nor may "Apache" appear in their name, without
 * prior written permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */
 package org.apache.jmeter.visualizers;

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.*;
import java.net.*;
import javax.mail.*;
import javax.mail.internet.*;
//import javax.activation.* ;
import java.util.Properties;
import org.apache.jmeter.util.*;
import org.apache.jmeter.samplers.SampleResult;
import org.apache.jmeter.samplers.Sampler;

/*
 * TODO :
 * - Create a subpanel for other visualizers
 * - connect to the properties.
 * - Get the specific URL that is failing.
 * - add a seperate interface to collect the thrown failure messages.
 * -
 * - suggestions ;-)
 */

/************************************************************
 *  This class implements a visualizer that mails a message when an error
 *  occurs.
 *
 *@author     <a href="mailto:stuart@personalmd.com">Stuart Schmukler</a>
 *@created    $Date: 2001/08/16 23:36:57 $
 *@version    $Revision: 1.17 $ $Date: 2001/08/16 23:36:57 $
 ***********************************************************/
public class MailerVisualizer extends JPanel implements Visualizer, ActionListener
{

	String addressie;
	String from;
	String smtpHost;
	String failsubject;
	String successsubject;
	long failureCount = 0;
	long successCount = 0;
	long failureLimit = 2;
	long successLimit = 2;
	boolean failureMsgSent = false;
	boolean siteDown = false;
	boolean successMsgSent = false;

	Properties appProperties;

	JButton testerButton;
	JTextField addressField;
	JTextField fromField;
	JTextField smtpHostField;
	JTextField failsubjectField;
	JTextField successsubjectField;
	JTextField failureField;
	JTextField failureLimitField;
	JTextField successLimitField;

	//-----------

	/************************************************************
	 *  !ToDo (Constructor description)
	 ***********************************************************/
	public MailerVisualizer()
	{
		super();

		// Properties connection.
		this.appProperties = JMeterUtils.getJMeterProperties();

		// File panel
		JPanel mailerPanel = new JPanel();
		GridBagLayout g = new GridBagLayout();
		mailerPanel.setLayout(g);
		GridBagConstraints c = new GridBagConstraints();

		c.anchor = GridBagConstraints.NORTHWEST;
		c.insets = new Insets(0, 0, 0, 0);
		c.gridwidth = 1;
		mailerPanel.add(new JLabel("From:"));

		from = appProperties.getProperty("mailer.from");
		fromField = new JTextField(from, 25);
		fromField.setEditable(true);
		fromField.addActionListener(this);
		c.gridwidth = GridBagConstraints.REMAINDER;
		g.setConstraints(fromField, c);
		mailerPanel.add(fromField);

		c.anchor = GridBagConstraints.NORTHWEST;
		c.insets = new Insets(0, 0, 0, 0);
		c.gridwidth = 1;
		mailerPanel.add(new JLabel("Addressie(s):"));

		addressie = appProperties.getProperty("mailer.addressies");
		addressField = new JTextField(addressie, 25);
		addressField.setEditable(true);
		addressField.addActionListener(this);
		c.gridwidth = GridBagConstraints.REMAINDER;
		g.setConstraints(addressField, c);
		mailerPanel.add(addressField);

		c.gridwidth = 1;
		mailerPanel.add(new JLabel("SMTP Host:"));

		smtpHost = appProperties.getProperty("mailer.smtphost");
		smtpHostField = new JTextField(smtpHost, 25);
		smtpHostField.setEditable(true);
		smtpHostField.addActionListener(this);
		c.gridwidth = GridBagConstraints.REMAINDER;
		g.setConstraints(smtpHostField, c);
		mailerPanel.add(smtpHostField);

		c.gridwidth = 1;
		mailerPanel.add(new JLabel("Failure Subject:"));

		failsubject = appProperties.getProperty("mailer.failsubject");
		failsubjectField = new JTextField(failsubject, 25);
		failsubjectField.setEditable(true);
		failsubjectField.addActionListener(this);
		c.gridwidth = GridBagConstraints.REMAINDER;
		g.setConstraints(failsubjectField, c);
		mailerPanel.add(failsubjectField);

		c.gridwidth = 1;
		mailerPanel.add(new JLabel("Success Subject:"));

		successsubject = appProperties.getProperty("mailer.successsubject");
		successsubjectField = new JTextField(successsubject, 25);
		successsubjectField.setEditable(true);
		successsubjectField.addActionListener(this);
		c.gridwidth = GridBagConstraints.REMAINDER;
		g.setConstraints(successsubjectField, c);
		mailerPanel.add(successsubjectField);

		c.gridwidth = 1;
		mailerPanel.add(new JLabel("Failure Limit:"));

		try
		{
			failureLimit =
					Long.parseLong(appProperties.getProperty("mailer.failurelimit"));
		}
		catch (NumberFormatException ex)
		{
			// Ignore any garbage
		}
		failureLimitField = new JTextField(Long.toString(failureLimit), 6);
		failureLimitField.setEditable(true);
		failureLimitField.addActionListener(this);
		c.gridwidth = GridBagConstraints.REMAINDER;
		g.setConstraints(failureLimitField, c);
		mailerPanel.add(failureLimitField);

		c.gridwidth = 1;
		mailerPanel.add(new JLabel("Success Limit:"));

		try
		{
			successLimit =
					Long.parseLong(appProperties.getProperty("mailer.successlimit"));
		}
		catch (NumberFormatException ex)
		{
			// Ignore any garbage
		}
		successLimitField = new JTextField(Long.toString(successLimit), 6);
		successLimitField.setEditable(true);
		successLimitField.addActionListener(this);
		c.gridwidth = GridBagConstraints.REMAINDER;
		g.setConstraints(successLimitField, c);
		mailerPanel.add(successLimitField);

		testerButton = new JButton("Test Mail");
		testerButton.addActionListener(this);
		testerButton.setEnabled(true);
		c.gridwidth = 1;
		g.setConstraints(testerButton, c);
		mailerPanel.add(testerButton);

		c.gridwidth = 1;
		mailerPanel.add(new JLabel("Failures:"));
		failureField = new JTextField(6);
		failureField.setEditable(false);
		failureField.addActionListener(this);
		c.gridwidth = GridBagConstraints.REMAINDER;
		g.setConstraints(failureField, c);
		mailerPanel.add(failureField);

		this.add(mailerPanel);
	}

	/************************************************************
	 *  !ToDoo (Method description)
	 *
	 *@return    !ToDo (Return description)
	 ***********************************************************/
	public synchronized boolean isFailing()
	{
		return (failureCount > failureLimit);
	}

	/************************************************************
	 *  !ToDoo (Method description)
	 *
	 *@return    !ToDo (Return description)
	 ***********************************************************/
	public JPanel getControlPanel()
	{
		return this;
	}

	/************************************************************
	 *  !ToDo (Method description)
	 *
	 *@param  e  !ToDo (Parameter description)
	 ***********************************************************/
	public void actionPerformed(ActionEvent e)
	{
		try
		{
			JComponent c = (JComponent)e.getSource();
			if (c == addressField)
			{
				this.addressie = this.addressField.getText();
				System.out.println("AddressField=" + addressField.getText());
			}
			else if (c == fromField)
			{
				this.from = this.fromField.getText();
				System.out.println("FromField=" + fromField.getText());
			}
			else if (c == smtpHostField)
			{
				this.smtpHost = this.smtpHostField.getText();
				System.out.println("smtpHostField=" + smtpHostField.getText());
			}
			else if (c == failsubjectField)
			{
				this.failsubject = this.failsubjectField.getText();
				System.out.println("failsubjectField=" + failsubjectField.getText());
			}
			else if (c == successsubjectField)
			{
				this.successsubject = this.successsubjectField.getText();
				System.out.println("successsubjectField=" + successsubjectField.getText());
			}
			else if (c == failureLimitField)
			{
				this.failureLimit = Long.parseLong(this.failureLimitField.getText());
				System.out.println("failureLimitField=" + failureLimitField.getText());
			}
			else if (c == successLimitField)
			{
				this.successLimit = Long.parseLong(this.successLimitField.getText());
				System.out.println("successLimitField=" + successLimitField.getText());
			}
			else if (c == testerButton)
			{
				System.out.println("### Test To:  " + this.addressie + ", " +
						"Via:  " + this.smtpHost + ", " +
						"Fail Subject:  " + this.failsubject + ", " +
						"Success Subject:  " + this.successsubject);

				String testMessage = ("### Test To:  " + this.addressie + ", " +
						"Via:  " + this.smtpHost + ", " +
						"Fail Subject:  " + this.failsubject + ", " +
						"Success Subject:  " + this.successsubject);

				Vector addressVector = newAddressVector(addressie);
				sendMail(from, addressVector, "Testing addressies", testMessage, smtpHost);
			}
			else
			{
				// do nothing!
			}
		}
		catch (Exception ex)
		{
			JOptionPane.showMessageDialog(this, ex, "Error", JOptionPane.ERROR_MESSAGE);
		}
	}


	/************************************************************
	 *  !ToDo (Method description)
	 *
	 *@param  theAddressie  !ToDo (Parameter description)
	 *@return               !ToDo (Return description)
	 ***********************************************************/
	public synchronized Vector newAddressVector(String theAddressie)
	{
		Vector addressVector = new Vector();
		String addressSep = ", ";

		StringTokenizer next = new StringTokenizer(theAddressie, addressSep);

		while (next.hasMoreTokens())
		{
			String theToken = next.nextToken();

			if (theToken.indexOf("@") > 0)
			{
				addressVector.addElement(theToken);
			}
		}

		return addressVector;
	}


	/************************************************************
	 *  !ToDo (Method description)
	 *
	 *@param  sample  !ToDo (Parameter description)
	 ***********************************************************/
	public synchronized void add(SampleResult sample)
	{

		// -1 is the code for a failed sample.
		//
		if (sample.getValue(SampleResult.SUCCESS) == null ||
				!((Boolean)sample.getValue(SampleResult.SUCCESS)).booleanValue())
		{
			failureCount++;
		}
		else
		{
			successCount++;
		}

		if (this.isFailing() && !siteDown && !failureMsgSent)
		{
			// Display ...
			failureField.setText(Long.toString(failureCount));
			repaint();

			// Send the mail ...
			Vector addressVector = newAddressVector(addressie);
			sendMail(from, addressVector, failsubject, "URL Failed: " +
					sample.getValue(SampleResult.SAMPLE_LABEL), smtpHost);
			siteDown = true;
			failureMsgSent = true;
			successCount = 0;
		}

		if (siteDown && (sample.getTime() != -1) & !successMsgSent)
		{
			// Display ...
			failureField.setText(Long.toString(failureCount));
			repaint();

			// Send the mail ...
			if (successCount > successLimit)
			{
				Vector addressVector = newAddressVector(addressie);
				sendMail(from, addressVector, successsubject, "URL Restarted: " +
						sample.getValue(SampleResult.SAMPLE_LABEL), smtpHost);
				siteDown = false;
				successMsgSent = true;
			}
		}

		if (successMsgSent && failureMsgSent)
		{
			clear();
		}
	}

	/************************************************************
	 *  !ToDo (Method description)
	 ***********************************************************/
	public synchronized void clear()
	{
		failureCount = 0;
		successCount = 0;
		siteDown = false;
		successMsgSent = false;
		failureMsgSent = false;

		failureField.setText(Long.toString(failureCount));
		repaint();
	}

	/************************************************************
	 *  !ToDo (Method description)
	 *
	 *@return    !ToDo (Return description)
	 ***********************************************************/
	public String toString()
	{
		return "E-Mail Notification";
	}

	//-----------
	//function to send a mail to list mailaddresses

	/************************************************************
	 *  !ToDo (Method description)
	 *
	 *@param  from      !ToDo (Parameter description)
	 *@param  vEmails   !ToDo (Parameter description)
	 *@param  subject   !ToDo (Parameter description)
	 *@param  attText   !ToDo (Parameter description)
	 *@param  SMTPHost  !ToDo (Parameter description)
	 ***********************************************************/
	public static synchronized void sendMail(String from,
			Vector vEmails,
			String subject,
			String attText,
			String SMTPHost)
	{
		try
		{
			String host = SMTPHost;
			boolean debug = Boolean.valueOf(host).booleanValue();
			InetAddress remote = InetAddress.getByName(host);

			InternetAddress[] address = new InternetAddress[vEmails.size()];
			for (int k = 0; k < vEmails.size(); k++)
			{
				address[k] = new InternetAddress(vEmails.elementAt(k).toString());
			}

			// create some properties and get the default Session
			Properties props = new Properties();
			props.put("mail.smtp.host", host);
			Session session = Session.getDefaultInstance(props, null);
			session.setDebug(debug);

			// create a message
			Message msg = new MimeMessage(session);
			msg.setFrom(new InternetAddress(from));
			msg.setRecipients(Message.RecipientType.TO, address);
			msg.setSubject(subject);
			msg.setText(attText);
			Transport.send(msg);
			System.out.println("Mail sent successfully!!");
		}
		catch (UnknownHostException e1)
		{
			System.out.println("NxError:Invalid Mail Server " + e1);
			System.exit(1);
		}
		catch (Exception e)
		{
			e.printStackTrace();
			System.exit(1);
		}
	}
}
