/*
 * ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2001 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 * if any, must include the following acknowledgment:
 * "This product includes software developed by the
 * Apache Software Foundation (http://www.apache.org/)."
 * Alternately, this acknowledgment may appear in the software itself,
 * if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" and
 * "Apache JMeter" must not be used to endorse or promote products
 * derived from this software without prior written permission. For
 * written permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 * "Apache JMeter", nor may "Apache" appear in their name, without
 * prior written permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

package org.apache.jmeter.visualizers;

import java.awt.*;
import java.util.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;

import org.apache.jmeter.reporters.ResultCollector;
import org.apache.jmeter.gui.*;
import org.apache.jmeter.util.JMeterUtils;
import org.apache.jmeter.samplers.Clearable;

/************************************************************
 *  Allows the tester to view the textual response from sampling an Entry. This
 *  also allows to "single step through" the sampling process via a nice
 *  "Continue" button.
 *
 *@author     $Author: khammond $
 *@created    $Date: 2001/10/07 23:41:00 $
 *@version    $Revision: 1.17 $ $Date: 2001/10/07 23:41:00 $
 ***********************************************************/
public class ViewResultsVisualizer extends JPanel implements ModelSupported, ActionListener,Clearable
{
	private JTextArea display;
	private JButton next;
	private JTextField label;
	private ResultCollector model;

	/************************************************************
	*  !ToDo (Constructor description)
	***********************************************************/
	public ViewResultsVisualizer()
	{
		super();
	}

	public void setModel(Object model)
	{
		this.model = (ResultCollector)model;
		this.model.setListener(this);
		init();
	}

	private void init()
	{
		this.setLayout(new VerticalLayout(5, VerticalLayout.LEFT, VerticalLayout.TOP));

		// MAIN PANEL
		JPanel mainPanel = new JPanel();
		Border margin = new EmptyBorder(10, 10, 5, 10);
		mainPanel.setBorder(margin);
		mainPanel.setLayout(new VerticalLayout(5, VerticalLayout.LEFT));

		// TITLE
		JLabel panelTitleLabel = new JLabel(JMeterUtils.getResString("view_results_title"));
		Font curFont = panelTitleLabel.getFont();
		int curFontSize = curFont.getSize();
		curFontSize += 4;
		panelTitleLabel.setFont(new Font(curFont.getFontName(), curFont.getStyle(), curFontSize));
		mainPanel.add(panelTitleLabel);

		// NAME
		mainPanel.add(new NamePanel(model));

		// INNER PANEL
		JPanel innerPanel = new JPanel();
		innerPanel.setPreferredSize(new Dimension(640, 480));
		label = new JTextField();
		display = new JTextArea();
		next = new JButton("Next");
		next.addActionListener(this);
		innerPanel.setLayout(new GridBagLayout());
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.gridx = gbc.gridy = 0;
		gbc.fill = gbc.BOTH;
		gbc.weightx = 1;
		gbc.weighty = 0;
		innerPanel.add(label, gbc.clone());

		gbc.gridy++;
		gbc.weighty = 1;
		innerPanel.add(new JScrollPane(display), gbc.clone());

		gbc.gridy++;
		gbc.fill = gbc.NONE;
		gbc.weightx = gbc.weighty = 0;
		gbc.anchor = gbc.CENTER;
		innerPanel.add(next, gbc.clone());

		mainPanel.add(innerPanel);
		this.add(mainPanel);
	}

	/************************************************************
	*  !ToDoo (Method description)
	*
	*@return    !ToDo (Return description)
	***********************************************************/
	public void updateGui()
	{
		if(label.getText().equals(""))
		{
			model.next();
			label.setText(model.getSampleLabel());
			display.setText(model.getTextResponse());
		}
	}

	/************************************************************
	*  !ToDo (Method description)
	***********************************************************/
	public void clear()
	{
		label.setText("");
		display.setText("");
	}

	/************************************************************
	*  !ToDo (Method description)
	*
	*@param  e  !ToDo (Parameter description)
	***********************************************************/
	public void actionPerformed(ActionEvent e)
	{
		label.setText("");
		updateGui();
	}

	/************************************************************
	*  !ToDo (Method description)
	*
	*@return    !ToDo (Return description)
	***********************************************************/
	public String toString()
	{
		return "Show text response from sample";
	}
}
